
/**************************************************************
 *	         Micro-GT mini Direct Control I/O                 *
 *                 G-Tronic Robotics                          *
 *                 beta 1 (17/07/2011)                        *
 *        Questo programma  sviluppato per PIC16F876A        *
 *               piattaforma Micro-GT mini                    *
 *                                                            *
 *                           Scopo del programma:             *
 *   Controllare via seriale 8 comandi di ON/OFF da inviare   *
 *   alla Micro-GT mini con la seguente legenda               *
 *   Nota: Firmware utilizzabile con Micro-GT versatile IDE   *
 *                                                            *
 *     Comandi di accensione:                                 *
 *     LED0    @I000inv  RB0 ->ON                             *
 *     LED1    @L000inv  RB1 ->ON                             *
 *     LED2    @M000inv  RB2 ->ON                             *
 *     LED3    @N000inv  RB3 ->ON                             *
 *     LED4    @O000inv  RB4 ->ON                             *
 *     LED5    @P000inv  RB5 ->ON                             *
 *     LED6    @Q000inv  RB6 ->ON                             *
 *     LED7    @R000inv  RB7 ->ON                             *
 *                                                            *
 *     Comandi di spegnimento:                                *
 *     LED0    @S000inv  RB0 ->OFF                            *
 *     LED1    @T000inv  RB1 ->OFF                            *
 *     LED2    @U000inv  RB2 ->OFF                            *
 *     LED3    @V000inv  RB3 ->OFF                            *
 *     LED4    @Z000inv  RB4 ->OFF                            *
 *     LED5    @X000inv  RB5 ->OFF                            *
 *     LED6    @Y000inv  RB6 ->OFF                            *
 *     LED7    @J000inv  RB7 ->OFF                            *
 *                                                            *
 *     RESET   @A000inv  resetta tutte le uscite              *
 *     quit    @B000inv  PORTB=0                              *
 *                                                            *
 *  Importante: Le stringhe inviate sono di tipo impulsivo da *
 *              parte dell'interfaccia .net che genera il     *
 *              comando di OFF                                *
 *************************************************************/

#include <pic.h>		//libreria standard di riconoscimento hardware 
#include "delay.h"      //libreria potenziata versione 2010 di MicroChip per creare ritardi

#define LED0    RB0 
#define LED1    RB1 
#define LED2    RB2 
#define LED3    RB3 
#define LED4    RB4 
#define LED5    RB5 
#define LED6    RB6 
#define LED7    RB7
#define RESET	0  

extern void Par_ser(void);	//predichiarazione funzione scansione contenuto stringhe
extern void Var_Init(void);	//predichiarazione funzione inizzializzazione variabili

unsigned char C1,C2,C3,C4,RxPos,Flags; 	//dichiarazione variabili contenute in stringa e di riconoscimento stringa 


main(void)		//inizia la funzione principale
{

TRISA = 0x00;	    // tutte uscite per evitare interferenze
TRISB = 0x00; 		// uscite comandi
TRISC = 0xf8;		// bit 6-7 Seriale
PORTA = 0x00;		// azzera le porte
PORTB = 0x00;	    // azzera il PORT B
PORTC = 0x00;



   Var_Init();		//inizializza le variabili di trasmissione seriale

ADCON1= 0x87;		// PORTA all digital mode

//Inizializzazione uart
TXSTA= 0x20;		// TX abilitato
RCSTA= 0X90;		// Registro RX USART
BRGH = 1;		    // USART alta velocit
SPBRG= 25;		    // Generatore di Baud Rate (25=9600 baud)

//Inizializzazione TIMER1
T1CON   = 0;		// TIMER 1 resettato
T1CKPS1 = 0,T1CKPS0=0;	// TIMER1 prescaler = 1
T1OSCEN = 0;		// TIMER1 external oscillator disable
TMR1CS  = 0;		// TIMER1 internal clock (fosc/4)
TMR1ON  = 0;		// TIMER1 in STOP

//Inizializzazione interrupt
INTCON = 0;		// Interrupt disabilitato
PIE1   = 0;		// Interrupt di periferica disabilitati
GIE    = 1;		// Global Interrupt Enable
PEIE   = 1;		// Periferal Interrupt Enable
RCIE   = 1;		// USART Receive Interrupt Enable



for (;;){
   if (Flags==1) Par_ser();
   DelayMs(2);			// Delay aggiuntivo
}  
 }


void interrupt SerChar(void)  //prototipo ricezione
{
  if (RCREG==0x40 | RxPos > 6)  //se arriva la chiocciola dimensiona la stringa
   {
   C1=0,C2=0,C3=0,C4=0;		//azzero la stringa
   RxPos=0,Flags=0;
   }
  if (RxPos==1) C1=RCREG;	//ricezione della stringa comandi
  if (RxPos==2) C2=RCREG; 
  if (RxPos==3) C3=RCREG;
  if (RxPos==4) C4=RCREG;
  if ((RxPos==5) && (RCREG==0x0d)) Flags=1; // <CR> terminator
  RxPos++;
}

void Var_Init(void){
RxPos=0;
Flags=0;
}	

void
Par_ser(void)		// Analisi comandi
{
 Flags=0;    
 
 
                       
 switch (C1){
   	case 0x49: 	//carattere I Accensione LED 0 
	LED0 = 1;
	break;
	case 0x53: 	//carattere S Spegnimento LED 0
	LED0 = 1;
	break;
		
    
   	case 0x4C:   //carattere L Accensione LED 1
  	LED1 = 1; 
  	break;
  	case 0x54:  //carattere T Spegnimento del LED 1
   	LED1 = 0;
	break;
 
   
   	case 0x4D: 	//carattere M Accensione LED 2
  	LED2 = 1;
 	break;
   	case 0x55: 	//carattere U Spegnimento LED 2   	
   	LED2 = 0;
    break;

   	case 0x4E:	//carattere N Accensione LED 3 
   	LED3 = 1;
	break;
	case 0x56:	//carattere V Spegnimento LED 3 
   	LED3 = 0;
	break;
   
   	case 0x4F: 	//carattere O Accensione LED 4 
   	LED4  = 1;  
	break;
	case 0x5A: 	//carattere Z Spegnimento LED 4 
   	LED4 = 0;
   	break;
   
   	case 0x50:	//carattere P Accensione LED 5 
	LED5 = 1; 
	break;
   	case 0x58:	//carattere X Spegnimento LED 5	
   	LED5 = 0;
   	break;
  
   	case 0x51: 	//carattere Q Accensione LED 6
   	LED6 = 1; 
   	break;
   	case 0x59: 	//carattere Y Spegnimento LED 6   
   	LED6 = 0;
 	break;
 	
   	case 0x52: 	//carattere R Accensione LED 7   
   	LED7 = 1; 
 	break;
   	case 0x4A: 	//carattere J Spegnimento LED 7
   	LED7=0;
     break;

	case 0x41:	//carattere A resetta il PORT B
	PORTB=RESET;
	break;
	case 0x42:  //carattere B inviato prima di uscire dall'interfaccia
	PORTB=RESET;
	break;   
  }
}
